// Sample code: load two images, extract feature vectors and compare the similarity
// This sample is similar to face_recognition.cpp, and shows how to call setImage() with an image buffer pointer.
// First two images of the same identity are compared. Then the similarity score of two different identities is computed.
// Build on Linux: g++ -o face_recognition_image_buffer --std=c++11 ./face_recognition_image_buffer.cpp -I ../ -L ../trueface_sdk -l tf -fopenmp
// Build on macOS: g++ -o face_recognition_image_buffer --std=c++11 ./face_recognition_image_buffer.cpp -I ../ -L ../trueface_sdk -l tf

#include "trueface_sdk/tf_sdk.h"
#define STB_IMAGE_IMPLEMENTATION
#include "3rd_party_libs/stb_image.h"
#include <iostream>
#include <vector>

int main() {
    Trueface::SDK tfSdk;
    // TODO: replace the string with your license code.
    bool valid = tfSdk.setLicense("${LICENSE_CODE}");

    if (valid == false) {
        std::cout<<"Error: the provided license is invalid."<<std::endl;
        return 1;
    }

    // Load the first image using STB_image.
    int width, height, channels;
    uint8_t* rgb_image = stbi_load("../images/brad_pitt_1.jpg", &width, &height, &channels, 3);

    // Pass the image array to Trueface SDK.
    Trueface::ErrorCode errorCode = tfSdk.setImage(rgb_image, width, height, Trueface::ColorCode::rgb);
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image"<<std::endl;
        return 1;
    }

    bool foundFace;
    Trueface::Faceprint faceprint1;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint1, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Error: Unable to detect face in image"<<std::endl;
        return 0;
    }
    // Release the allocated image array after the feature vector has been extracted.
    stbi_image_free(rgb_image);

    // Load the second image and extract the feature vector.
    rgb_image = stbi_load("../images/brad_pitt_2.jpg", &width, &height, &channels, 3);

    errorCode = tfSdk.setImage(rgb_image, width, height, Trueface::ColorCode::rgb);
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image 2"<<std::endl;
        return 1;
    }
    Trueface::Faceprint faceprint2;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint2, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Error: Unable to detect face in image"<<std::endl;
        return 0;
    }
    // Release the allocated image array after the feature vector has been extracted.
    stbi_image_free(rgb_image);

    // Compute the similarity between the two face images.
    float similarityScore;
    float matchProbability;
    tfSdk.getSimilarity(faceprint1, faceprint2, matchProbability, similarityScore);

    std::cout<<"Similarity score of same identity images: "<<similarityScore<<std::endl;
    std::cout<<"Match probability of same identity images: "<<matchProbability<<std::endl;

    // Load the image of a different identity and extract the feature vector.
    rgb_image = stbi_load("../images/tom_cruise_1.jpg", &width, &height, &channels, 3);

    errorCode = tfSdk.setImage(rgb_image, width, height, Trueface::ColorCode::rgb);
    if (errorCode != Trueface::ErrorCode::NO_ERROR) {
        std::cout<<"Error: could not load the image 2"<<std::endl;
        return 1;
    }
    Trueface::Faceprint faceprint3;
    errorCode = tfSdk.getLargestFaceFeatureVector(faceprint3, foundFace);
    if (errorCode != Trueface::ErrorCode::NO_ERROR || !foundFace) {
        std::cout<<"Error: Unable to detect face in image"<<std::endl;
        return 0;
    }
    // Release the allocated image array after the feature vector has been extracted.
    stbi_image_free(rgb_image);

    // Compute the similarity between the images of different identities.
    tfSdk.getSimilarity(faceprint1, faceprint3, matchProbability, similarityScore);

    std::cout<<"Similarity score of two different identities: "<<similarityScore<<std::endl;
    std::cout<<"Match probability of two different identities: "<<matchProbability<<std::endl;

    return 0;
}